#include "sis.h"
#include "io_int.h"

/* The subroutines util_strchch and util_map_pathname are from
   polis/hw/hw_util.c in the polis project; the rest of the file is
   polis/misc/io/write_pld.c from the polis project. */

/*
 * util_strchch
 *
 * Changes every occurrence of char a into char b in string s
 *
 */
char *
util_strchch(s, a, b)
char *s, a, b;
{
  int i, n;

  n = strlen(s);
  for (i=0; i<n && s[i]!='\0'; i++) {
      if (s[i] == a)
          s[i] = b;
  }
  return s;
}


char *
util_map_pathname(name)
char *name;
{
   static char str[512];
   char *new_name, head = 'z';

   new_name = name;
   if (name[0] == '*') {
       head = 'e';
       new_name++;
   }

   sprintf(str, "%c_%s", head, new_name);
   new_name = str;

   /* remove annoying characters */
   util_strchch(new_name, '[', '_');
   util_strchch(new_name, ']', '_');
   util_strchch(new_name, '<', 'b');
   util_strchch(new_name, '>', 'b');
   util_strchch(new_name, '{', '_');
   util_strchch(new_name, '}', '_');
   util_strchch(new_name, '/', 'o');
   util_strchch(new_name, '\\', '_');
   util_strchch(new_name, '*', 'e');
   util_strchch(new_name, '$', 'S');

   return new_name;
}

#define node_to_input(j, s)   \
  switch(j){                   \
     case 0:                   \
       strcpy(s, "A");         \
       break;                  \
     case 1:                   \
       strcpy(s, "B");       \
       break;                  \
     case 2:                   \
       strcpy(s, "C");       \
       break;                  \
     case 3:                   \
       strcpy(s,"D");       \
       break;                  \
     case 4:                   \
       strcpy(s,"E");       \
       break;                  \
     default:                  \
       (void)fprintf(sisout, "Error shouldnt be here\n"); \
       abort();               \
   }

static char *A = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";


#define node_short_name(n) n->short_name
extern char *node_decide_name();


static int
po_fanout_count(n)
node_t *n;
{
    node_t *fo;
    lsGen gen;
    int po_count;

    po_count = 0;
    foreach_fanout(n, gen, fo){
        if(node_function(fo) == NODE_PO) po_count++;
    }
    return po_count;
}

char *
node_decide_name(node, short_name)
node_t *node;
int short_name;
{
    extern char *io_name();

    return util_map_pathname(io_name(node, short_name));
}

static void
write_pds_sop(fp, node, short_name)
FILE *fp;
node_t *node;
int short_name;
{
    int first_cube, first_literal;
    int i, j;
    node_cube_t cube;
    node_literal_t literal;
    node_t *fanin;
    extern int io_node_should_be_printed();
      
    if (io_node_should_be_printed(node) == 0) {
	   return;
       }

    (void)fprintf(fp, "%s = ", node_decide_name(node, short_name));
    switch(node_function(node)){
    case NODE_0:
        (void)fprintf(fp, "GND\n");
	return;
    case NODE_1:
	(void)fprintf(fp, "VCC\n");
	return;
    case NODE_PO:
	(void)fprintf(fp, "%s\n", 
		      node_decide_name(node_get_fanin(node, 0), short_name));
	return;
      }
    first_cube = 1;
    for(i = node_num_cube(node)-1; i >= 0; i--){
        if(!first_cube){
	    (void)fprintf(fp, " + ");
	}
	cube = node_get_cube(node, i);
	
	first_literal = 1;
	foreach_fanin(node, j, fanin){
	    literal = node_get_literal(cube, j);
	    switch(literal){
	    case ZERO:
	    case ONE:
	      if(!first_literal){
		  (void)fprintf(fp, " * ");
	      }
	      if(literal == ZERO) (void)fprintf(fp, "/");
	      (void)fprintf(fp, "%s", node_decide_name(fanin, short_name));
	      first_literal = 0;
	      break;
	    }
	
	}
	first_cube = 0;
    }
    (void)fprintf(fp, "\n");
}


static void
write_palasm(fp, network, short_name, combinational_flag) 
FILE *fp;
network_t *network;
int short_name, combinational_flag; /* combinational_flag = 1 =>  do not write out latch equations */
{
    node_t *po, *pi, *node;
    lsGen gen;
    int i;
#ifdef SIS
    latch_t *latch;
    node_t *input, *output;
    char *input_name,  *output_name;
#endif

    (void)fprintf(fp, "CHIP dummy LCA\n");
    (void)fprintf(fp,"\n");
    
    (void)fprintf(fp, ";PINLIST\n");
    i = 0;
    
    foreach_primary_input(network, gen, pi){
#ifdef SIS
        if (!combinational_flag) {
            if (!network_is_real_pi(network, pi)) continue;
        }
#endif
        (void)fprintf(fp, "%s ", node_decide_name(pi, short_name));
	    i++;
	if( i > 9){
	    (void)fprintf(fp, "\n");
	    i = 0;
	}
    }

    foreach_primary_output(network, gen, po){
#ifdef SIS
        if (!combinational_flag) {
            if (!network_is_real_po(network, po)) continue;
        }
#endif
        (void)fprintf(fp, "%s ", node_decide_name(po, short_name));
        i++;
	if( i > 9){
	    (void)fprintf(fp, "\n");
	    i = 0;
	}
    }
    
    (void)fprintf(fp, "\n");
#ifdef SIS
    if (!combinational_flag) {
        if (network_num_latch(network) > 0) {
            (void) fprintf(fp, "clock\n");
        }
    }
#endif
    (void)fprintf(fp, "EQUATIONS\n");
    
    foreach_node(network, gen, node){
        (void)write_pds_sop(fp, node, short_name);
    }
#ifdef SIS
    if (!combinational_flag) {
        /* write out the flip-flop equations */
        /*-----------------------------------*/
        foreach_latch(network, gen, latch) {
            input = latch_get_input(latch);
            output = latch_get_output(latch);
            input_name = node_decide_name(input, short_name);
            output_name = node_decide_name(output, short_name);        
            fprintf(fp, "%s := %s\n", output_name, input_name);
            fprintf(fp, "%s.CLKF = clock\n", output_name);
        }
    }
#endif
}

int
com_pds(network, argc, argv)
network_t **network;
int argc;
char **argv;
{
    int c, short_name, combinational_flag;
    FILE *fp;
    /* Init */
    error_init();
    util_getopt_reset();
    debug = 0;
    short_name = 0;
    combinational_flag = 0;
    name_mode = LONG_NAME_MODE;
    /* Default params */

    util_getopt_reset();
    while ((c = util_getopt(argc, argv, "cds")) != EOF) {
	switch (c) {
	case'c':
	  combinational_flag = 1;
	  break;
	case'd':
	  debug = 1;
	  break;  
	case's':
	  short_name = 1;
	  name_mode = SHORT_NAME_MODE;
	  break;
        default:
	  goto usage;
	}
    }
    if(argc - util_optind == 0){
        fp = com_open_file("-", "w", NIL(char *), /* silent */0);
    }else if((argc - util_optind) == 1) {
        fp = com_open_file(argv[util_optind], "w", NIL(char *), /* silent */0);
    } else {
        goto usage;
    }

    (void)write_palasm(fp, *network, short_name, combinational_flag);
    if((argc - util_optind) == 1){
        fclose(fp);
    }
    return 0;

    usage:
        (void)fprintf(sisout,"usage: %s [-csd] [filename]\n", argv[0]);
	(void)fprintf(sisout,"-c\t\ttreat the network as a combinational one - do not print latch eqns\n");
	(void)fprintf(sisout,"-s\t\tuse network short-name\n");
    return 1;
}

static node_t * 
latch_single_fanout (node)
node_t *node;
{
	lsGen gen;
	node_t *fanout, *result;

	result = NIL(node_t);
	foreach_fanout(node, gen, fanout){
		if (network_latch_end (fanout) != NIL(node_t)) {
			result = fanout;
		}
		else if (node_type (fanout) == INTERNAL) {
			return NIL(node_t);
		}
	}
	return result;
}

static node_t * 
latch_fanout (node)
node_t *node;
{
	lsGen gen;
	node_t *fanout;

	foreach_fanout(node, gen, fanout){
		if (network_latch_end (fanout) != NIL(node_t)) {
			return fanout;
		}
	}
	return NIL(node_t);
}


static void
print_equation_map(fp, node, index)
FILE *fp;
node_t *node;
int *index;
{
    int first_cube, first_literal;
    int i, j;
    node_cube_t cube;
    node_literal_t literal;
    node_t *fanin;
    char s[1];

    if(node_num_fanin(node) > 5) {
        (void)fprintf(siserr, "Error in size of CLB\n");
		return;
    }
    
    first_cube = 1;
    for(i = node_num_cube(node)-1; i >= 0; i--){
        if(!first_cube){
            (void)fprintf(fp, " + ");
	  }
        cube = node_get_cube(node, i);

        first_literal = 1;
	foreach_fanin(node, j, fanin){
            literal = node_get_literal(cube, j);
            switch(literal){
            case ZERO:
	    case ONE:
              if(!first_literal){
                  (void)fprintf(fp, " * ");
	      }
	      if(literal == ZERO) (void)fprintf(fp, "~");
		  if (index[j] > 4) {
			fprintf (siserr, "ERROR: node %s has more than 5 inputs\n",
				node_name (node));
			return;
		  }
	      node_to_input(index[j], s);
	      (void)fprintf(fp, "%s", s);
	      first_literal = 0;
	      break;
	    }
	}
	first_cube = 0;
    }
}

static char *
node_pld_name(node, vdd, gnd)
node_t *node;
st_table *vdd, *gnd;
{
    extern char *io_name();
	node_t *latch;

	if (st_is_member (gnd, (char*) node)) {
		return "GND";
	}
	if (st_is_member (vdd, (char*) node)) {
		return "VDD";
	}
	/* if PO fanout, get the PI */
	latch = latch_single_fanout (node);
	if (latch != NIL(node_t)) {
		node = network_latch_end (latch);
	}
	if (st_is_member (gnd, (char*) node)) {
		return "GND";
	}
	if (st_is_member (vdd, (char*) node)) {
		return "VDD";
	}
	/* if PO, get the PI */
	if (node_type (node) == PRIMARY_OUTPUT &&
		network_latch_end (node) != NIL(node_t)) {
		node = network_latch_end (node);
	}
	if (st_is_member (gnd, (char*) node)) {
		return "GND";
	}
	if (st_is_member (vdd, (char*) node)) {
		return "VDD";
	}
    return util_map_pathname (io_name(node, /*short_name*/ 0));
}

static void
encode_function(node, fp)
node_t *node;
FILE *fp;
{
    int first_cube, first_literal;
    int i, j;
    node_cube_t cube;
    node_literal_t literal;
    node_t *fanin;
    char s[1];

    if(node_num_fanin(node) > 5) {
        (void)fprintf(siserr, "Error in size of CLB\n");
		return;
    }
    first_cube = 1;
    for(i = node_num_cube(node)-1; i >= 0; i--){
        if(!first_cube){
            (void)fprintf(fp, " + ");
          }
        cube = node_get_cube(node, i);

        first_literal = 1;
        foreach_fanin(node, j, fanin){
            literal = node_get_literal(cube, j);
            switch(literal){
            case ZERO:
	    case ONE:
		if(!first_literal){
		    (void)fprintf(fp, " * ");
		}
		if(literal == ZERO) (void)fprintf(fp, "~");
		(void)fprintf(fp, "%c", A[j]);
		first_literal = 0;
		break;
            }
        }
        first_cube = 0;
    }
    (void)fprintf(fp,")\n");
}
  
 
static void
write_map(fp,  network, short_name, part, merge_file)
FILE *fp;
network_t *network;
int short_name;
char *part;
FILE *merge_file;
{
    node_t *node, *fanin, *node2, **new_fanin, **new_fanin2, *latch, *latch2;
	node_t *single_latch, *single_latch2, *fanout, *new_node;
    char s[1], buf[256], name1[256], name2[256];
    lsGen gen, gen1;
    int i, found, new_nin, a, b, c, d, e, empty, index[80];
	pset_family newf, newg;
	st_table *first, *second, *gnd, *vdd;

	first = st_init_table (st_ptrcmp, st_ptrhash); 
	second = st_init_table (st_ptrcmp, st_ptrhash); 
	vdd = st_init_table (st_ptrcmp, st_ptrhash); 
	gnd = st_init_table (st_ptrcmp, st_ptrhash); 
	if (merge_file != NULL) {
		found = 0;
		while (fgets (buf, sizeof(buf), merge_file) != NULL) {
			if (! strcmp (buf, "Merging two CLB's into one CLB\n")) {
				found = 1;
			}
			else if (! strncmp (buf, "# of CLB's =", 11)) {
				found = 0;
			}
			else if (found) {
				if (sscanf (buf, "Merge node %s and %s", name1, name2) == 2) {
					node = network_find_node (network, name1);
					if (node == NIL(node_t)) {
						fprintf (siserr, "cannot find paired CLB %s\n",
							name1);
						return;
					}
					if (node_function (node) == NODE_0 ||
						node_function (node) == NODE_1) {
						continue;
					}
					node2 = network_find_node (network, name2);
					if (node2 == NIL(node_t)) {
						fprintf (siserr, "cannot find paired CLB %s\n",
							name2);
						return;
					}
					if (node_function (node2) == NODE_0 ||
						node_function (node2) == NODE_1) {
						continue;
					}
					st_insert (first, (char*) node, (char *) node2);
					st_insert (second, (char*) node2, (char *) node);
				}
			}
		}
	}

    (void)fprintf(fp,"LCANET, 2\n");
    (void)fprintf(fp, "PROG, XNFMAP: created from %s:\n", 
		network_name(network));
    (void)fprintf(fp, "PART, %s\n", part);
    (void)fprintf(fp,"PWR, 0, GND\n");
    (void)fprintf(fp,"PWR, 1, VCC\n");
    
    
    foreach_node(network, gen, node){
		if (node_type (node) == PRIMARY_INPUT) {
			foreach_fanout (node, gen1, fanout) {
				if (node_type (fanout) == PRIMARY_OUTPUT) {
					new_node = node_literal (node, 1);
					network_add_node (network, new_node);
					assert (node_patch_fanin (fanout, node, new_node));
				}
			}
		}
		else if (node_function (node) == NODE_0) {
			st_insert (gnd, (char*) node, NIL(char));
		}
		else if (node_function (node) == NODE_1) {
			st_insert (vdd, (char*) node, NIL(char));
		}
	}
    foreach_node(network, gen, node){
		if (st_is_member (second, (char*) node) ||
			st_is_member (gnd, (char*) node) ||
			st_is_member (vdd, (char*) node)) {
			/* we will deal with it when we meet its companion */
			continue;
		}
        if (node_type(node) != INTERNAL) {
			continue;
		}
		if (! st_lookup (first, (char*) node, (char**) &node2)) {
			node2 = NIL(node_t);
		}
		/* separate latch and node if it has also internal fanout (sigh...) */
		latch = latch_single_fanout (node);
		latch2 = NIL(node_t);
		if (node2 != NIL(node_t)) {
			latch2 = latch_single_fanout (node2);
		}
		/* we could try and optimally pair orphaned latches here */
		single_latch = NIL(node_t);
		single_latch2 = NIL(node_t);
		if (latch == NIL(node_t)) {
			single_latch = latch_fanout (node);
		}
		if (node2 != NIL(node_t) && latch2 == NIL(node_t)) {
			single_latch2 = latch_fanout (node2);
		}
		if (single_latch != NIL(node_t) || single_latch2 != NIL(node_t)) {
			/* DO NOT use node or node2 within this if !! */
			if (single_latch == NIL(node_t)) {
				single_latch = single_latch2;
				single_latch2 = NIL(node_t);
			}
			(void)fprintf(fp,"SYM, %s, CLB\n", 
				node_pld_name(single_latch, vdd, gnd));
			if (single_latch2 != NIL(node_t)) {
				(void)fprintf(fp,"CFG, Base FG\n");
			}
			else {
				(void)fprintf(fp,"CFG, Base F\n");
			}
			(void)fprintf(fp, "CFG, Config RSTDIR: ENCLK: CLK:K ");
			if (single_latch2 != NIL(node_t)) {
				(void)fprintf(fp,"X:QX DX:F Y:QY DY:G ");
			}
			else {
				(void)fprintf(fp, "X:QX DX:F Y: DY: ");
			}
			(void)fprintf(fp, "\n");
			(void)fprintf(fp,"CFG, Equate F=(A)\n");
			if (single_latch2 != NIL(node_t)) {
				(void)fprintf(fp,"CFG, Equate G=(B)\n");
			}
			(void)fprintf(fp, "PIN, A, I, %s\n", 
				node_pld_name(node_get_fanin (single_latch, 0), vdd, gnd));
			if (single_latch2 != NIL(node_t)) {
				(void)fprintf(fp, "PIN, B, I, %s\n", 
					node_pld_name(node_get_fanin (single_latch2, 0), vdd, gnd));
			}
			(void)fprintf(fp, "PIN, K, I, GCLK\n");
			(void)fprintf(fp, "PIN, X, O, %s\n", 
				node_pld_name(network_latch_end(single_latch), vdd, gnd));
			if (single_latch2 != NIL(node_t)) {
				(void)fprintf(fp, "PIN, Y, O, %s\n", 
					node_pld_name(network_latch_end(single_latch2), vdd, gnd));
			}
			(void)fprintf(fp,"END\n");
		}

		/* print the header */
		(void)fprintf(fp,"SYM, %s, CLB\n", node_pld_name(node, vdd, gnd));
		if (node2 != NIL(node_t)) {
			(void)fprintf(fp,"CFG, Base FG\n");
		}
		else {
			(void)fprintf(fp,"CFG, Base F\n");
		}
		(void)fprintf(fp, "CFG, Config RSTDIR: ENCLK: ");
		if (latch != NIL(node_t) || latch2 != NIL(node_t)) {
			(void)fprintf(fp, "CLK:K ");
		}
		else {
			(void)fprintf(fp, "CLK: ");
		}
		if (latch != NIL(node_t)) {
			(void)fprintf(fp, "X:QX DX:F ");
		}
		else {
			(void)fprintf(fp, "X:F DX: ");
		}
		if (node2 != NIL(node_t)) {
			if (latch2 != NIL(node_t)) {
				(void)fprintf(fp, "Y:QY DY:G ");
			}
			else {
				(void)fprintf(fp, "Y:G DY: ");
			}
		}
		else {
			(void)fprintf(fp, "Y: DY: ");
		}
		(void)fprintf(fp, "\n");
		for (i = 0; i < node_num_fanin (node); i++) {
			index[i] = i;
		}
		if (node2 != NIL(node_t)) {
			make_common_base (node, node2, &new_fanin, &new_nin, &newf, &newg);
			new_fanin2 = ALLOC (node_t *, new_nin);
			a = b = c = d = e = -1;
			for (i = 0; i < new_nin; i++) {
				if (a < 0 &&
					node_get_fanin_index (node, new_fanin[i]) >= 0 && 
					node_get_fanin_index (node2, new_fanin[i]) >= 0) {
					a = i;
				}
				if (d < 0 &&
					node_get_fanin_index (node, new_fanin[i]) >= 0 && 
					node_get_fanin_index (node2, new_fanin[i]) < 0) {
					d = i;
				}
				if (e < 0 &&
					node_get_fanin_index (node, new_fanin[i]) < 0 && 
					node_get_fanin_index (node2, new_fanin[i]) >= 0) {
					e = i;
				}
			}
			for (i = 0; i < new_nin; i++) {
				if (a != i && d != i && e != i) {
					if (a >= 0) {
						if (b >= 0) {
							if (c >= 0) {
								if (d >= 0) {
									e = i;
								}
								else {
									d = i;
								}
							}
							else {
								c = i;
							}
						}
						else {
							b = i;
						}
					}
					else {
						a = i;
					}
				}
			}
			for (i = 0; i < new_nin; i++) {
				if (a == i) {
					index[a] = 0;
				}
				else if (b == i) {
					index[b] = 1;
				}
				else if (c == i) {
					index[c] = 2;
				}
				else if (d == i) {
					index[d] = 3;
				}
				else if (e == i) {
					index[e] = 4;
				}
			}
			for (i = 0; i < new_nin; i++) {
				new_fanin2[i] = new_fanin[i];
			}
			node_replace_internal (node, new_fanin, new_nin, newf);
			node_replace_internal (node2, new_fanin2, new_nin, newg);
		}
		(void)fprintf(fp,"CFG, Equate F=(");
		print_equation_map(fp, node, index);
		(void)fprintf(fp,")\n");
		if (node2 != NIL(node_t)) {
			(void)fprintf(fp,"CFG, Equate G=(");
			print_equation_map(fp, node2, index);
			(void)fprintf(fp,")\n");
		}
		foreach_fanin(node, i, fanin){
		    if (index[i] > 4) {
				fprintf (siserr, "ERROR: node %s has more than 5 inputs\n",
					node_name (node));
				return;
		    }
			node_to_input(index[i], s);
			(void)fprintf(fp, "PIN, %s, I, %s\n", s, 
				node_pld_name(fanin, vdd, gnd));
		}
		if (latch != NIL(node_t) || latch2 != NIL(node_t)) {
			(void)fprintf(fp, "PIN, K, I, GCLK\n");
		}
		if (latch != NIL(node_t)) {
			(void)fprintf(fp, "PIN, X, O, %s\n", 
				node_pld_name(latch, vdd, gnd));
		}
		else {
			(void)fprintf(fp, "PIN, X, O, %s\n", 
				node_pld_name(node, vdd, gnd));
		}
		if (node2 != NIL(node_t)) {
			/* its fanin has been printed by X already */
			if (latch2 != NIL(node_t)) {
				(void)fprintf(fp, "PIN, Y, O, %s\n", 
					node_pld_name(latch2, vdd, gnd));
			}
			else {
				(void)fprintf(fp, "PIN, Y, O, %s\n", 
					node_pld_name(node2, vdd, gnd));
			}
		}
		if (node2 != NIL(node_t)) {
			node_minimum_base (node);
			node_minimum_base (node2);
		}
		(void)fprintf(fp,"END\n");
    }
	/* buffers */
	(void)fprintf(fp, "SYM, GCLK0, GCLK\n");
	(void)fprintf(fp, "PIN,I,I,I0\n");
	(void)fprintf(fp, "PIN,O,O,GCLK\n");
	(void)fprintf(fp, "END\n");
	i = 1;
	foreach_primary_input(network, gen, node) {
		if (! network_is_real_pi (network, node)) {
			continue;
		}
		(void)fprintf(fp, "SYM, IBUF%d, IBUF\n", i);
		(void)fprintf(fp, "PIN,I,I,I%d\n", i);
		(void)fprintf(fp, "PIN,O,O,%s\n",
			node_pld_name(node, vdd, gnd));
		(void)fprintf(fp, "END\n");
		i++;
	}
	foreach_primary_output(network, gen, node) {
		if (! network_is_real_po (network, node)) {
			continue;
		}
		(void)fprintf(fp, "SYM, OBUF%d, OBUF\n", i);
		(void)fprintf(fp, "PIN,O,O,O%d\n", i);
		(void)fprintf(fp, "PIN,I,I,%s\n",
			node_pld_name(node_get_fanin (node, 0), vdd, gnd));
		(void)fprintf(fp, "END\n");
		i++;
	}
	(void)fprintf(fp, "EXT, I0, I\n");
	i = 1;
	foreach_primary_input(network, gen, node) {
		if (! network_is_real_pi (network, node)) {
			continue;
		}
		(void)fprintf(fp, "EXT, I%d, I\n", i);
		i++;
	}
	foreach_primary_output(network, gen, node) {
		if (! network_is_real_po (network, node)) {
			continue;
		}
		(void)fprintf(fp, "EXT, O%d, O\n", i);
		i++;
	}
    (void)fprintf(fp,"EOF\n");
	st_free_table (first);
	st_free_table (second);
	st_free_table (vdd);
	st_free_table (gnd);
}

int
com_map(network, argc, argv)
network_t **network;
int argc;
char **argv;
{

    int c, short_name;
    FILE *fp, *merge_file;
    char *buff, *part, *merge_name;


    /* Init */
    error_init();
    util_getopt_reset();
    debug = 0;
    short_name = 0;
	part = NIL(char);
	merge_name =  "merged_nodes";
    name_mode = LONG_NAME_MODE;
    /* Default params */


    while((c = util_getopt(argc, argv, "dsp:m:")) != EOF) {
	switch (c) {
	case'd':
	  debug = 1;
	  break;  
	case'p':
	  part = util_optarg;
	  break;  
	case'm':
	  merge_name = util_optarg;
	  break;  
	case's':
	  short_name = 1;
	  name_mode = SHORT_NAME_MODE;
	  break;
        default:
	  goto usage;
	}
    }
    if(argc - util_optind == 0){
        buff = ALLOC(char, strlen(network_name(*network))+5);
	strcpy(buff, network_name(*network));
        strcat(buff, ".map"); 
        fp = com_open_file(buff, "w", NIL(char *), 0);
	FREE(buff);
    }else if((argc - util_optind) == 1) {
        buff = ALLOC(char, strlen(argv[util_optind])+5);
	strcpy(buff, argv[util_optind]);
        fp = com_open_file(buff, "w", NIL(char *), 0);
	FREE(buff);
    } else {
        goto usage;
    }
	if (part == NIL(char)) {
		goto usage;
	}
	merge_file = fopen (merge_name, "r");
 
    (void)write_map(fp, *network, short_name, part, merge_file);
    if((argc - util_optind) == 1){
        fclose(fp);
    }
	if (merge_file != NULL) {
		fclose (merge_file);
	}
    return 0;

    usage:
        (void)fprintf(sisout,"usage: %s [-s] [-m merge_file_name] -p part_name [filename]\n", argv[0]);
	(void)fprintf(sisout,"-s\t\tuse network short-name\n");
	(void)fprintf(sisout,"-m\t\txl_merge -o file (default merged_nodes)\n");
	(void)fprintf(sisout,"-p\t\tXILINX part name (must be specified)\n");
	(void)fprintf(sisout,"(file name defaults to network name .map)\n");
    return 1;
}



